// $Id: regcache.cpp,v 1.4 2002/01/11 09:27:56 ccr Exp $
// regcache.cpp -- register cache space to be freed at end of program

// regcache offers a way to register allocated memory which is not freed
// for good reason,  just to let a debugger know it does not mean
// a memory leak.  Before you register memory with it, make sure you
// know it doesn't leak!

#include "regcache.h"

#include <list>
#if defined(WIN32) || (defined(__GNUC__) && __GNUC__ > 2)
#define list std::list
#endif

// Do not instatiate this class, it's meant as a singleton object.
// The only reason o make this a class is the restricted number
// of functions which can be registered via atexit().
template <class T>
class _regcache {
private:
	list<T*> _tofree;
public:
	_regcache() {};
	~_regcache() {
		list<T*>::iterator it;
		for(it=_tofree.begin(); it!=_tofree.end(); ++it)
			free(*it);
	};
	T* reg_cache(T* a) { _tofree.push_back(a); return a; };
	T* unreg_cache(T* a) { _tofree.remove(a); return a; };
};


static _regcache<void> _cache;

void* register_as_cache_mem(void *x) { return _cache.reg_cache(x); }
void* unregister_cache_mem(void *x) { return _cache.unreg_cache(x); }
void kpse_free(void *x) { unregister_cache_mem(x); free(x); }
